@@ -1,10 +1,52 @@
 from dexy.common import OrderedDict
 from dexy.plugins.process_filters import SubprocessFilter
+from dexy.plugins.process_filters import SubprocessInputFilter
 import dexy.exceptions
 import json
 import os
 import shutil
 
+class PythonInput(SubprocessInputFilter):
+    """
+    Runs python code and passes input.
+    """
+    EXECUTABLE = 'python'
+    ALIASES = ['pyin']
+    VERSION_COMMAND = 'python --version'
+    OUTPUT_EXTENSIONS = ['.txt']
+
+class Pdfcrop(SubprocessFilter):
+    """
+    Runs the PDFcrop script http://pdfcrop.sourceforge.net/
+    """
+    ALIASES = ['pdfcrop']
+    EXECUTABLE = 'pdfcrop'
+    INPUT_EXTENSIONS = ['.pdf']
+    OUTPUT_EXTENSIONS = ['.pdf']
+
+class CalibreFilter(SubprocessFilter):
+    """
+    Invokes ebook-convert command line tool (part of calibre) to generate various output formats (including .mobi for Kindle)
+
+    http://manual.calibre-ebook.com/cli/ebook-convert.html
+    """
+    ADD_NEW_FILES = False
+    ALIASES = ['calibre', 'ebook']
+    EXECUTABLE = "ebook-convert"
+    INPUT_EXTENSIONS = ['.epub', '.html']
+    OUTPUT_EXTENSIONS = ['.mobi']
+    PATH_EXTENSIONS = ['/Applications/calibre.app/Contents/MacOS']
+    VERSION_COMMAND = "ebook-convert --version"
+
+    def command_string(self):
+        args = {
+            'prog' : self.executable(),
+            'args' : self.command_line_args() or "",
+            'input_file' : self.input_filename(),
+            'output_file' : self.output_filename()
+        }
+        return "%(prog)s %(input_file)s %(output_file)s %(args)s" % args
+
 class LyxFilter(SubprocessFilter):
     """
     Invokes lyx on the command line. By default generates a latex file.
@@ -30,6 +72,9 @@ class LyxFilter(SubprocessFilter):
         return "%(prog)s %(format_string)s %(args)s %(input_file)s" % args
 
 class PandocFilter(SubprocessFilter):
+    """
+    Converts documents using pandoc.
+    """
     EXECUTABLE = "pandoc"
     VERSION_COMMAND = "pandoc --version"
     ALIASES = ['pandoc']
@@ -54,8 +99,11 @@ class PandocFilter(SubprocessFilter):
         return "%(prog)s %(args)s %(script_file)s -o %(output_file)s" % args
 
 class EspeakFilter(SubprocessFilter):
+    """
+    Runs espeak text to speech.
+    """
     EXECUTABLE = "espeak"
-    INPUT_EXTENSIONS = [".txt"]
+    INP2UT_EXTENSIONS = [".txt"]
     OUTPUT_EXTENSIONS = [".wav"]
     ALIASES = ['espeak']
 
@@ -70,6 +118,9 @@ class EspeakFilter(SubprocessFilter):
         return "%(prog)s %(args)s -w %(output_file)s %(script_file)s" % args
 
 class AsciidocFilter(SubprocessFilter):
+    """
+    Converts ASCIIDOC input.
+    """
     VERSION_COMMAND = "asciidoc --version"
     EXECUTABLE = "asciidoc"
     INPUT_EXTENSIONS = [".txt"]
@@ -104,6 +155,9 @@ class AsciidocFilter(SubprocessFilter):
         return "%(prog)s -b %(backend)s %(args)s -o %(outfile)s %(infile)s" % args
 
 class BlackWhitePdfFilter(SubprocessFilter):
+    """
+    Converts color pdf to black and white.
+    """
     EXECUTABLE = "gs"
     ALIASES = ['bw', 'bwconv']
     INPUT_EXTENSIONS = [".pdf"]
@@ -156,6 +210,12 @@ class Pdf2ImgSubprocessFilter(SubprocessFilter):
         shutil.copyfile(page_path, self.output_filepath())
 
 class Pdf2JpgSubprocessFilter(Pdf2ImgSubprocessFilter):
+    """
+    Converts a PDF file to a jpg image using ghostscript.
+
+    Returns the image generated by page 1 of the PDF by default, the
+    'page' parameter can be used to specify other pages.
+    """
     ALIASES = ['pdf2jpg']
     GS_DEVICE = 'jpeg'
     OUTPUT_EXTENSIONS = ['.jpg']
@@ -228,6 +288,9 @@ class RagelRubySubprocessFilter(SubprocessFilter):
         return "%s %s -o %s" % (self.executable(), wf, of)
 
 class Rd2PdfFilter(SubprocessFilter):
+    """
+    Generates a pdf from R documentation file.
+    """
     INPUT_EXTENSIONS = [".Rd"]
     OUTPUT_EXTENSIONS = [".pdf", ".dvi"]
     EXECUTABLE = 'R CMD Rd2pdf'
@@ -246,6 +309,7 @@ class Rd2PdfFilter(SubprocessFilter):
 
 class RBatchFilter(SubprocessFilter):
     """Runs R code in batch mode."""
+    ADD_NEW_FILES = True
     ALIASES = ['rintbatch']
     EXECUTABLE = 'R CMD BATCH --quiet --no-timing'
     INPUT_EXTENSIONS = ['.txt', '.r', '.R']
@@ -253,6 +317,10 @@ class RBatchFilter(SubprocessFilter):
     VERSION_COMMAND = "R --version"
 
 class RIntBatchSectionsFilter(SubprocessFilter):
+    """
+    Experimental filter to run R in sections without using pexpect.
+    """
+    ADD_NEW_FILES = True
     ALIASES = ['rintmock']
     EXECUTABLE = 'R CMD BATCH --quiet --no-timing'
     INPUT_EXTENSIONS = ['.txt', '.r', '.R']
@@ -305,6 +373,7 @@ class RIntBatchSectionsFilter(SubprocessFilter):
 
 class ROutputBatchFilter(SubprocessFilter):
     """Runs R code in batch mode. Uses the --slave flag so doesn't echo commands, just returns output."""
+    ADD_NEW_FILES = True
     ALIASES = ['rout', 'routbatch']
     EXECUTABLE = 'R CMD BATCH --vanilla --quiet --slave --no-timing'
     INPUT_EXTENSIONS = ['.R', '.r', '.txt']
@@ -368,6 +437,9 @@ class HtLatexFilter(SubprocessFilter):
         return """%(prog)s %(script_file)s "%(args)s" "%(tex4htargs)s" "%(t4htargs)s" "%(latexargs)s" """ % args
 
 class AbcFilter(SubprocessFilter):
+    """
+    Runs abc to convert abc music notation to one of the available output formats.
+    """
     ADD_NEW_FILES = False
     ALIASES = ['abc']
     EXECUTABLE = 'abcm2ps'
@@ -418,6 +490,9 @@ class AbcFilter(SubprocessFilter):
             self.add_new_files()
 
 class AbcMultipleFormatsFilter(SubprocessFilter):
+    """
+    Runs abc to convert abc music notation to all of the available output formats.
+    """
     ALIASES = ['abcm']
     INPUT_EXTENSIONS = ['.abc']
     OUTPUT_EXTENSIONS = ['.json']
@@ -472,3 +547,27 @@ class AbcMultipleFormatsFilter(SubprocessFilter):
                 output[ext] = f.read()
 
         self.output().set_data(json.dumps(output))
+
+class RdConv(SubprocessFilter):
+    """Convert R documentation to other formats."""
+    EXECUTABLE = "R CMD Rdconv"
+    VERSION_COMMAND = "R CMD Rdconv -v"
+    INPUT_EXTENSIONS = ['.Rd']
+    OUTPUT_EXTENSIONS = ['.txt', '.html', '.tex', '.R']
+    ALIASES = ['rdconv']
+    EXTENSION_TO_FORMAT = {
+        '.txt' : 'txt',
+        '.html' : 'html',
+        '.tex' : 'latex',
+        '.R' : 'example'
+    }
+
+    def command_string(self):
+        args = {
+                'exe' : self.executable(),
+                'clargs' : self.command_line_args() or "",
+                'fmt' : self.EXTENSION_TO_FORMAT[self.artifact.ext],
+                'input' : self.input_filename(),
+                'output' : self.output_filename()
+                }
+        return "%(exe)s %(clargs)s --type=%(fmt)s --output=%(output)s %(input)s" % args
